From 9fe1f4aa4f4a0add6621db5b87dfdbe8589515eb Mon Sep 17 00:00:00 2001
From: Wei Gao <gaowei66@huawei.com>
Date: Mon, 23 Nov 2020 00:15:03 -0500
Subject: [PATCH 2/2] machine_config: add unit parser in json format.

1. Add memory unit (M, m, G, g) parser in json file format.

Signed-off-by: Wei Gao <gaowei66@huawei.com>
---
 machine_manager/src/config/machine_config.rs | 40 ++++++++++----------
 1 file changed, 19 insertions(+), 21 deletions(-)

diff --git a/machine_manager/src/config/machine_config.rs b/machine_manager/src/config/machine_config.rs
index fbe4e15..880228f 100644
--- a/machine_manager/src/config/machine_config.rs
+++ b/machine_manager/src/config/machine_config.rs
@@ -16,7 +16,8 @@ extern crate serde_json;
 use serde::{Deserialize, Serialize};
 
 use super::errors::{ErrorKind, Result};
-use crate::config::{CmdParams, ConfigCheck, ParamOperation, VmConfig};
+use crate::config::{CmdParams, ConfigCheck, Param, ParamOperation, VmConfig};
+use util::num_ops::round_down;
 
 const DEFAULT_CPUS: u8 = 1;
 const DEFAULT_MEMSIZE: u64 = 128;
@@ -82,8 +83,11 @@ impl MachineConfig {
             machine_config.nr_cpus = value["vcpu_count"].to_string().parse::<u8>().unwrap();
         }
         if value.get("mem_size") != None {
-            machine_config.mem_config.mem_size =
-                value["mem_size"].to_string().parse::<u64>().unwrap();
+            let mut param = Param {
+                param_type: String::new(),
+                value: value["mem_size"].to_string().replace("\"", ""),
+            };
+            machine_config.mem_config.mem_size = unit_conversion(&mut param);
         }
         if value.get("mem_path") != None {
             machine_config.mem_config.mem_path =
@@ -139,25 +143,9 @@ impl VmConfig {
     pub fn update_memory(&mut self, mem_config: String) {
         let cmd_params: CmdParams = CmdParams::from_str(mem_config);
         if let Some(mut mem_size) = cmd_params.get("") {
-            if mem_size.value_replace_blank("M") || mem_size.value_replace_blank("m") {
-                self.machine_config.mem_config.mem_size =
-                    get_inner(mem_size.value_to_u64().checked_mul(M));
-            } else if mem_size.value_replace_blank("G") || mem_size.value_replace_blank("g") {
-                self.machine_config.mem_config.mem_size =
-                    get_inner(mem_size.value_to_u64().checked_mul(G));
-            } else {
-                self.machine_config.mem_config.mem_size = mem_size.value_to_u64();
-            }
+            self.machine_config.mem_config.mem_size = unit_conversion(&mut mem_size)
         } else if let Some(mut mem_size) = cmd_params.get("size") {
-            if mem_size.value_replace_blank("M") || mem_size.value_replace_blank("m") {
-                self.machine_config.mem_config.mem_size =
-                    get_inner(mem_size.value_to_u64().checked_mul(M));
-            } else if mem_size.value_replace_blank("G") || mem_size.value_replace_blank("g") {
-                self.machine_config.mem_config.mem_size =
-                    get_inner(mem_size.value_to_u64().checked_mul(G));
-            } else {
-                self.machine_config.mem_config.mem_size = mem_size.value_to_u64();
-            }
+            self.machine_config.mem_config.mem_size = unit_conversion(&mut mem_size)
         }
     }
 
@@ -176,6 +164,16 @@ impl VmConfig {
     }
 }
 
+fn unit_conversion(origin_param: &mut Param) -> u64 {
+    if origin_param.value_replace_blank("M") || origin_param.value_replace_blank("m") {
+        get_inner(origin_param.value_to_u64().checked_mul(M))
+    } else if origin_param.value_replace_blank("G") || origin_param.value_replace_blank("g") {
+        get_inner(origin_param.value_to_u64().checked_mul(G))
+    } else {
+        get_inner(round_down(origin_param.value_to_u64(), M))
+    }
+}
+
 fn get_inner<T>(outer: Option<T>) -> T {
     if let Some(x) = outer {
         x
-- 
2.27.0

